# Wobbly Balls
# Copyright (C) 2010 Jeffrey Armstrong <jeff@rainbow-100.com>
# http://jeff.rainbow-100.com/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# A full version of the license terms is available in LICENSE.

import pygame
import math
from constants import *
from block import GColumn, PlayBlockGroup
import data

class ColumnGuides(pygame.sprite.Sprite):
    def __init__(self,rocker):
        pygame.sprite.Sprite.__init__(self)
        
        self.rocker = rocker
        self.rect = pygame.Rect(0,0,SCREEN_X,SCREEN_Y)
        self.base_rect = pygame.Rect(self.rect)

        self.angle = 0
        self.aup = True
        
        self.guide_length = COL_HEIGHT
        
        self.create_image()
        
    def create_image(self):
        self.image = pygame.Surface((self.rect.width,self.rect.height))
        self.image.convert()
        
        self.image.set_colorkey(pygame.Color('black'))
        self.image.fill(pygame.Color('black'))
                
        x = self.rocker.rect.left - self.rocker.rect.centerx
          
        for i in range(0,NUM_COLUMNS+1):
            
            xoffset = x*math.cos(math.radians(self.angle))
            yoffset = x*math.sin(math.radians(self.angle))
            
            start_point = (self.rocker.rect.centerx + xoffset, \
                           self.rocker.rect.centery + yoffset)
            
            xoffset = xoffset + self.guide_length*math.sin(math.radians(self.angle))
            yoffset = yoffset - self.guide_length*math.cos(math.radians(self.angle))
            
            end_point = (self.rocker.rect.centerx + xoffset, \
                         self.rocker.rect.centery + yoffset)
            
            pygame.draw.line(self.image,pygame.Color(128,128,128),start_point,end_point)
            
            x = x + BLOCK_X + 5        
        
    def set_angle(self,angle):
        self.angle = -1.0*angle
        self.aup = True
        
    def update(self):
        if self.aup:
            self.create_image()
            self.aup = False

class Rocker(pygame.sprite.Sprite):
    
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        
        self.rect = pygame.Rect(0,0,ROCKER_X,ROCKER_Y)
        self.rect.centerx = SCREEN_X/2
        self.rect.bottom = SCREEN_Y
        
        self.base_rect = pygame.Rect(self.rect)

        self.radius = ROCKER_Y/2
        
        self.create_image()
        
        self.angle = 0
        self.roll_dist = 0
        self.aup = True
        
        self.columns = []
        #centerx = self.rect.centerx - NUM_COLUMNS/2*(BLOCK_X+5)+BLOCK_X/2+3
        #centerx = -1.0*( NUM_COLUMNS/2*(BLOCK_X+5)+BLOCK_X/2+3 )
        centerx = -1.0*self.rect.width/2.0 + 3 + BLOCK_X/2.0
        for i in range(0,NUM_COLUMNS):
            self.columns.append(GColumn(centerx,SCREEN_Y-ROCKER_Y/2,self))
            centerx = centerx+BLOCK_X+5
            
        self.guides = ColumnGuides(self)
        
        self.block_rendering = pygame.sprite.RenderUpdates()
    
    def empty_all(self):
        for c in self.columns:
            c.empty()
        
    def create_image(self):
        self.image = pygame.Surface((ROCKER_X,ROCKER_Y))
        
        self.image.convert()
        
        self.image.set_colorkey(pygame.Color('black'))
        self.image.fill(pygame.Color('black'))
        
        self.image.set_clip(pygame.Rect(0,self.rect.height/2,self.rect.width,self.rect.height/2))
        pygame.draw.circle(self.image, pygame.Color(0,0,255,255), (ROCKER_X/2,ROCKER_Y/2), self.radius, 0)
        self.image.set_clip(None)
        
        self.base_image = self.image
        
        self.play_group = None
        
    def set_angle(self,angle):
        self.angle = angle
        self.aup = True
        self.guides.set_angle(angle)
        
        dist = -1.0*math.pi*self.base_image.get_width()*self.angle/360.0
        for c in self.columns:
            c.set_angle(angle)
        
    def new_play_group(self,level):
        self.play_group = PlayBlockGroup(level,self)
        
        self.block_rendering.add(self.play_group.blocks)

        return self.play_group
        
    def drop_play_group(self):
        if self.play_group == None:
            # This condition should not occur, but just return
            # "yea, it worked" in case it does happen
            return True
            
        # Check if possible
        if self.play_group.allowed():
            
            self.play_group.drop()
            self.play_group = None    
            
            return True
            
        else:
        
            return False

    def random_drop_play_group(self):
        if self.play_group != None:
            self.play_group.random_drop()
            
        self.play_group = None
        
    def do_rotate(self):
        r_tmp = pygame.Rect((0,0),self.base_image.get_size())
        self.image = pygame.transform.rotate(self.base_image,self.angle)
        
        r_tmp.center = (int(self.image.get_width()/2),int(self.image.get_height()/2))
        
        self.image = self.image.subsurface(r_tmp)
        
        # shift the rect as well
        self.roll_dist = math.pi*self.base_image.get_width()*self.angle/360.0
        
        self.rect = pygame.Rect(self.base_rect)
        self.rect.centerx = int(self.rect.centerx - self.roll_dist)
        
        #print self.rect.center
        
    def update(self):
        for col in self.columns:
            col.update()
            broken = col.broken_blocks()
            for b in broken:
                col.remove(b)
                self.block_rendering.remove(b)
        
        if self.aup:
            self.do_rotate()
            self.aup = False
            
        if self.play_group:
            self.play_group.update()

class Danger(pygame.sprite.Sprite):
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        
        self.start = None
        self.end = None
        
        self.create_image()
        
        self.rect = pygame.Rect((0,0),self.image.get_size())
        self.rect.centerx = SCREEN_X-10-self.rect.width/2
        self.rect.bottom = SCREEN_Y-10
        self.restart = False
        
    def on(self):
        if self.end:
            self.restart = True
        elif self.start == None:
            self.start = pygame.time.get_ticks()
            self.end = None
    
    def off(self):
        if self.start:
            self.start = None
            self.end = pygame.time.get_ticks()
    
    def create_image(self):
        font = pygame.font.Font(data.filepath(SCORE_FONT_NAME),DANGER_FONT_SIZE)
        render = font.render('Danger!',1,pygame.Color(255,0,0,255))
        self.image = pygame.Surface((render.get_width(),render.get_height()))
        self.image.fill(pygame.Color(0,0,0,0))
        self.image.set_colorkey(pygame.Color(0,0,0,0))
        self.image.blit(render,(0,0))
        self.image.set_alpha(0)
    
    def update(self):
        
        alpha = 0
        
        if self.start:
            fraction = float(pygame.time.get_ticks()-self.start)/float(DANGER_TIME)
            alpha = min(255,int(255.0*fraction))
        
        elif self.end:
            fraction = float(pygame.time.get_ticks()-self.end)/float(DANGER_TIME)
            alpha = min(255,int(255.0*(1.0-fraction)))
            if alpha < 0:
                self.end = None
                if self.restart:
                    self.restart = False
                    self.start = pygame.time.get_ticks()
        
        self.image.set_alpha(alpha)